@@ -1,8 +1,8 @@
-;; camlp4 ./pa_lispr.cmo pa_extend.cmo q_MLast.cmo pr_dump.cmo
-;; $Id: pa_lisp.ml,v 3.12 2001/06/17 12:02:10 ddr Exp $
+;; camlp5 ./pa_lispr.cmo pa_extend.cmo q_MLast.cmo pr_dump.cmo
+;; $Id: pa_lisp.ml 1222 2007-09-21 19:25:27Z deraugla $
+;; Copyright (c) INRIA 2007
 
 (open Pcaml)
-(open Stdpp)
 
 (type (choice 'a 'b) (sum (Left 'a) (Right 'b)))
 
@@ -63,13 +63,14 @@
 
 (value rec char
        (lambda len
-	 (parser
-	  (((` ''')) len)
-	  (((` x) s) (char (Buff.store len x) s)))))
+         (parser
+          (((` ''')) len)
+          (((` x) s) (char (Buff.store len x) s)))))
 
 (value quote
        (parser
-        (((` '\\') (len (char (Buff.store 0 '\\')))) (, "CHAR" (Buff.get len)))
+        (((` '\\') (len (char (Buff.store 0 '\\'))))
+         (, "CHAR" (Buff.get len)))
         (((` x) s) (char_or_quote_id x s))))
 
 (value rec
@@ -125,12 +126,12 @@
        (lambda (kwt (, con prm))
          (match con
                 ((or "CHAR" "EOI" "INT" "LIDENT" "QUOT" "STRING" "UIDENT") ())
-                (("ANTIQUOT") ())
+                ((or "ANTIQUOT" "ANTIQUOT_LOC") ())
                 (("")
                  (try (Hashtbl.find kwt prm)
                       (Not_found (Hashtbl.add kwt prm ()))))
                 (_ (raise
-                    (Token.Error
+                    (Plexing.Error
                      (^ "the constructor \""
                         (^ con "\" is not recognized by Plexer"))))))))
 
@@ -140,15 +141,20 @@
            (if (= prm "") con
              (^ con (^ " \"" (^ prm "\"")))))))
 
-(value lexer_make
+(value lexer_gmake
        (lambda ()
-         (let ((kwt (Hashtbl.create 89)))
+         (let ((kwt (Hashtbl.create 89))
+               (lexer2
+                (lambda (kwt (, s _ _))
+                  (let (((, t loc) (lexer kwt s)))
+                    (, t (Ploc.make_unlined loc))))))
            ({}
-            (Token.func (Token.lexer_func_of_parser (lexer kwt)))
-            (Token.using (lexer_using kwt))
-            (Token.removing (lambda))
-            (Token.tparse (lambda _ None))
-            (Token.text lexer_text)))))
+            (Plexing.tok_func (Plexing.lexer_func_of_parser (lexer2 kwt)))
+            (Plexing.tok_using (lexer_using kwt))
+            (Plexing.tok_removing (lambda))
+            (Plexing.tok_match Plexing.default_match)
+            (Plexing.tok_text lexer_text)
+            (Plexing.tok_comm None)))))
 
 ;; Building AST
 
@@ -160,7 +166,7 @@
 
 (value error_loc
        (lambda (loc err)
-         (raise_with_loc loc (Stream.Error (^ err " expected")))))
+         (Ploc.raise loc (Stream.Error (^ err " expected")))))
 (value error
        (lambda (se err)
          (let ((loc (match se
@@ -209,7 +215,7 @@
            (let ((e (expr_se se))) <:str_item< $exp:e$ >>))
           ((Sexpr loc (list (Satom _ Alid "module") (Satom _ Auid i) se))
            (let ((mb (module_binding_se se)))
-             <:str_item< module $i$ = $mb$ >>))
+             <:str_item< module $uid:i$ = $mb$ >>))
           ((Sexpr loc (list (Satom _ Alid "open") (Satom _ Auid s)))
            (let ((s (list s)))
              <:str_item< open $s$ >>))
@@ -222,7 +228,7 @@
                           ((list (Satom _ Alid "rec") :: sel) (, True sel))
                           ((_) (, False sel))))
                   (lbs (value_binding_se sel)))
-             <:str_item< value $rec:r$ $list:lbs$ >>))
+             <:str_item< value $flag:r$ $list:lbs$ >>))
           ((Sexpr loc _)
            (let ((e (expr_se se)))
              <:str_item< $exp:e$ >>))))
@@ -277,7 +283,7 @@
             ((list (Sexpr _ sel1) :: sel2)
              (let* ((lbs (List.map let_binding_se sel1))
                     (e (progn_se loc sel2)))
-               <:expr< let $rec:r$ $list:lbs$ in $e$ >>))
+               <:expr< let $flag:r$ $list:lbs$ in $e$ >>))
             ((list se :: _) (error se "let_binding"))
             ((_) (error_loc loc "let_binding")))))
   ((Sexpr loc (list (Satom _ Alid "let*") :: sel))
@@ -361,9 +367,9 @@
  (lambda loc
    (lambda_match
     ((Sexpr _ (list se1 se2))
-     (, (patt_se se1) None (expr_se se2)))
+     (, (patt_se se1) <:vala< None >> (expr_se se2)))
     ((Sexpr _ (list se1 sew se2))
-     (, (patt_se se1) (Some (expr_se sew)) (expr_se se2)))
+     (, (patt_se se1) <:vala< (Some (expr_se sew)) >> (expr_se se2)))
     (se (error se "match_case"))))
  label_expr_se
  (lambda loc
@@ -380,8 +386,7 @@
              (if (= i (String.length s))
                  (if (> i ibeg)
                      (expr_id loc (String.sub s ibeg (- i ibeg)))
-                   (raise_with_loc (, (- (+ (fst loc) i) 1)
-                                      (+ (fst loc) i))
+                   (Ploc.raise (Ploc.sub loc (- i 1) 1)
                                    (Stream.Error "expr expected")))
                (if (= ([] s i) '.')
                    (if (> i ibeg)
@@ -390,8 +395,7 @@
                                    (String.sub s ibeg (- i ibeg))))
                               (e2 (loop (+ i 1) (+ i 1))))
                          <:expr< $e1$ . $e2$ >>)
-                     (raise_with_loc (, (- (+ (fst loc) i) 1)
-                                        (+ (+ (fst loc) i) 1))
+                     (Ploc.raise (Ploc.sub loc (- i 1) 1)
                                      (Stream.Error "expr expected")))
                  (loop ibeg (+ i 1)))))))
          (loop 0 0))))
@@ -432,7 +436,7 @@
             (j (junk_fun loc))
             (k (ekont err)))
        <:expr< match $e$ $lid:strm_n$ with
-       [ Some $p$ $when:wo$ -> do { $j$ $lid:strm_n$ ; $skont$ }
+       [ Some $p$ $opt:wo$ -> do { $j$ $lid:strm_n$ ; $skont$ }
        | _ -> $k$ ] >>))
      ((Sexpr loc (list se1 se2))
       (let* ((p (patt_se se1))
@@ -497,8 +501,7 @@
          (if (= i (String.length s))
              (if (> i ibeg)
                  (patt_id loc (String.sub s ibeg (- i ibeg)))
-               (raise_with_loc (, (- (+ (fst loc) i) 1)
-                                  (+ (fst loc) i))
+               (Ploc.raise (Ploc.sub loc (- i 1) 1)
                                (Stream.Error "patt expected")))
            (if (= ([] s i) '.')
                (if (> i ibeg)
@@ -507,8 +510,7 @@
                                (String.sub s ibeg (- i ibeg))))
                           (p2 (loop (+ i 1) (+ i 1))))
                      <:patt< $p1$ . $p2$ >>)
-                 (raise_with_loc (, (- (+ (fst loc) i) 1)
-                                    (+ (+ (fst loc) i) 1))
+                 (Ploc.raise (Ploc.sub loc (- i 1) 1)
                                  (Stream.Error "patt expected")))
              (loop ibeg (+ i 1)))))))
      (loop 0 0)))
@@ -530,22 +532,27 @@
  type_declaration_list_se
  (lambda_match
   ((list se1 se2 :: sel)
-   (let (((, n1 tpl)
+   (let (((, n1 loc1 tpl)
           (match se1
-                 ((Sexpr _ (list (Satom _ Alid n) :: sel))
-                  (, n (List.map type_parameter_se sel)))
-                 ((Satom _ Alid n)
-                  (, n (list)))
+                 ((Sexpr _ (list (Satom loc Alid n) :: sel))
+                  (, n loc (List.map type_parameter_se sel)))
+                 ((Satom loc Alid n)
+                  (, n loc (list)))
                  ((se)
                   (error se "type declaration")))))
-     (list (, n1 tpl (ctyp_se se2) (list)) ::
-           (type_declaration_list_se sel))))
+     (let* ((empty (list))
+            (td ({} (MLast.tdNam (, loc1 <:vala< n1 >>))
+                    (MLast.tdPrm <:vala< tpl >>) (MLast.tdPrv <:vala< False >>)
+                    (MLast.tdDef (ctyp_se se2))
+                    (MLast.tdCon <:vala< empty >>))))
+          (list td :: (type_declaration_list_se sel)))))
   ((list) (list))
   ((list se :: _) (error se "type_declaration")))
  type_parameter_se
  (lambda_match
   ((Satom _ Alid s) (&& (>= (String.length s) 2) (= ([] s 0) '''))
-   (, (String.sub s 1 (- (String.length s) 1)) (, False False)))
+   (let ((s (String.sub s 1 (- (String.length s) 1))))
+      (, <:vala< s >> (, False False))))
   (se
    (error se "type_parameter")))
  ctyp_se
@@ -568,8 +575,7 @@
               (if (= i (String.length s))
                   (if (> i ibeg)
                       (ctyp_id loc (String.sub s ibeg (- i ibeg)))
-                    (raise_with_loc (, (- (+ (fst loc) i) 1)
-                                       (+ (fst loc) i))
+                    (Ploc.raise (Ploc.sub loc (- i 1) 1)
                                     (Stream.Error "ctyp expected")))
                 (if (= ([] s i) '.')
                     (if (> i ibeg)
@@ -577,15 +583,14 @@
                                     loc (String.sub s ibeg (- i ibeg))))
                                (t2 (loop (+ i 1) (+ i 1))))
                           <:ctyp< $t1$ . $t2$ >>)
-                      (raise_with_loc (, (- (+ (fst loc) i) 1)
-                                         (+ (+ (fst loc) i) 1))
+                      (Ploc.raise (Ploc.sub loc (- i 1) 1)
                                       (Stream.Error "ctyp expected")))
                   (loop ibeg (+ i 1)))))))
      (loop 0 0)))
  constructor_declaration_se
  (lambda_match
   ((Sexpr loc (list (Satom _ Auid ci) :: sel))
-   (, ci (List.map ctyp_se sel)))
+   (, loc <:vala< ci >> <:vala< (List.map ctyp_se sel) >>))
   (se
    (error se "constructor_declaration"))))
 
@@ -598,7 +603,7 @@
                      (let ((n (String.sub s 1 (- (String.length s) 1))))
                        (match sl
                               ((list (Satom _ Astring s))
-                               (MLast.StDir loc n (Some <:expr< $str:s$ >>)))
+                               (<:str_item< # $lid:n$ $str:s$ >>))
                               (_ (match ()))))
                    (str_item_se se)))
                 ((Sexpr loc _) (str_item_se se)))))
@@ -611,7 +616,7 @@
 (:= Pcaml.no_constructors_arity.val False)
 
 (progn
- (Grammar.Unsafe.reinit_gram gram (lexer_make ()))
+ (Grammar.Unsafe.gram_reinit gram (lexer_gmake ()))
  (Grammar.Unsafe.clear_entry interf)
  (Grammar.Unsafe.clear_entry implem)
  (Grammar.Unsafe.clear_entry top_phrase)
@@ -629,6 +634,9 @@
  (Grammar.Unsafe.clear_entry class_sig_item)
  (Grammar.Unsafe.clear_entry class_str_item))
 
+(:= Pcaml.parse_interf.val (Grammar.Entry.parse interf))
+(:= Pcaml.parse_implem.val (Grammar.Entry.parse implem))
+
 (value sexpr (Grammar.Entry.create gram "sexpr"))
 (value atom (Grammar.Entry.create gram "atom"))
 
