@@ -1,16 +1,6 @@
-(* camlp4r pa_extend.cmo q_MLast.cmo *)
-(***********************************************************************)
-(*                                                                     *)
-(*                             Camlp4                                  *)
-(*                                                                     *)
-(*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
-(*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
-(*  Automatique.  Distributed only by permission.                      *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id: pa_oop.ml,v 3.0 2001/04/19 07:51:31 ddr Exp $ *)
+(* camlp5r pa_extend.cmo q_MLast.cmo *)
+(* $Id: pa_oop.ml 1168 2007-09-15 19:35:16Z deraugla $ *)
+(* Copyright (c) INRIA 2007 *)
 
 open Pcaml;
 
@@ -20,7 +10,8 @@ type spat_comp =
   | SpStr of MLast.loc and MLast.patt ]
 ;
 type sexp_comp =
-  [ SeTrm of MLast.loc and MLast.expr | SeNtr of MLast.loc and MLast.expr ]
+  [ SeTrm of MLast.loc and MLast.expr
+  | SeNtr of MLast.loc and MLast.expr ]
 ;
 
 value strm_n = "strm__";
@@ -63,7 +54,7 @@ value rec stream_pattern loc epo e ekont =
       let (tst, p, wo, e) = stream_pattern_component skont spc in
       let ckont = ekont err in
       <:expr< match $tst$ with
-              [ Some $p$ $when:wo$ -> $e$ | _ -> $ckont$ ] >> ]
+              [ Some $p$ $opt:wo$ -> $e$ | _ -> $ckont$ ] >> ]
 ;
 
 value rec parser_cases loc =
@@ -114,7 +105,7 @@ EXTEND
   expr: LEVEL "expr1"
     [ [ "parser"; po = OPT ipatt; OPT "|"; pcl = LIST1 parser_case SEP "|" ->
           <:expr< $cparser loc po pcl$ >>
-      | "match"; e = expr; "with"; "parser"; po = OPT ipatt; OPT "|";
+      | "match"; e = SELF; "with"; "parser"; po = OPT ipatt; OPT "|";
         pcl = LIST1 parser_case SEP "|" ->
           <:expr< $cparser_match loc e po pcl$ >> ] ]
   ;
@@ -124,20 +115,21 @@ EXTEND
   ;
   stream_patt:
     [ [ spc = stream_patt_comp -> [(spc, None)]
-      | spc = stream_patt_comp; ";"; sp = LIST1 stream_patt_comp_err SEP ";" ->
+      | spc = stream_patt_comp; ";";
+        sp = LIST1 stream_patt_comp_err SEP ";" ->
           [(spc, None) :: sp]
       | (* empty *) -> [] ] ]
   ;
   stream_patt_comp_err:
     [ [ spc = stream_patt_comp;
-        eo = OPT [ "?"; e = expr LEVEL "expr1" -> e ] ->
+        eo = OPT [ "??"; e = expr LEVEL "expr1" -> e ] ->
           (spc, eo) ] ]
   ;
   stream_patt_comp:
-    [ [ "'"; p = patt; eo = OPT [ "when"; e = (expr LEVEL "expr1") -> e ] ->
-         SpTrm loc p eo
-     | p = patt; "="; e = (expr LEVEL "expr1") -> SpNtr loc p e
-     | p = patt -> SpStr loc p ] ]
+    [ [ "'"; p = patt; eo = OPT [ "when"; e = expr LEVEL "expr1" -> e ] ->
+          SpTrm loc p eo
+      | p = patt; "="; e = expr LEVEL "expr1" -> SpNtr loc p e
+      | p = patt -> SpStr loc p ] ]
   ;
   ipatt:
     [ [ i = LIDENT -> <:patt< $lid:i$ >> ] ]
