@@ -1,16 +1,6 @@
-(* camlp4r pa_ifdef.cmo *)
-(***********************************************************************)
-(*                                                                     *)
-(*                             Camlp4                                  *)
-(*                                                                     *)
-(*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
-(*                                                                     *)
-(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
-(*  Automatique.  Distributed only by permission.                      *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id: odyl_main.ml,v 3.1 2001/05/30 04:24:36 ddr Exp $ *)
+(* camlp5r pa_macro.cmo *)
+(* $Id: odyl_main.ml 1169 2007-09-16 03:41:14Z deraugla $ *)
+(* Copyright (c) INRIA 2007 *)
 
 value go = ref (fun () -> ());
 value name = ref "odyl";
@@ -35,7 +25,7 @@ Arg.current.val := first_arg_no_load () - 1;
 (* Load files in core *)
 
 value find_in_path path name =
-  if not (Filename.is_implicit name) then
+  if Filename.basename name <> name then
     if Sys.file_exists name then name else raise Not_found
   else
     let rec try_dir =
@@ -52,15 +42,16 @@ exception Error of string and string;
 
 value nolib = ref False;
 value initialized = ref False;
-value path = ref [];
+value path = ref ([] : list string);
 
 value loadfile file =
-  ifdef OPT then
+  IFDEF OPT THEN
     raise (Error file "native-code program cannot do a dynamic load")
-  else do {
+  ELSE do {
     if not initialized.val then do {
-      ifdef OPT then ()
-      else do { Dynlink.init (); Dynlink.allow_unsafe_modules True };
+      IFDEF OPT THEN ()
+      ELSE do { Dynlink.init (); Dynlink.allow_unsafe_modules True }
+      END;
       initialized.val := True
     }
     else ();
@@ -75,6 +66,7 @@ value loadfile file =
     try Dynlink.loadfile fname with
     [ Dynlink.Error e -> raise (Error fname (Dynlink.error_message e)) ]
   }
+  END
 ;
 
 value directory d = path.val := [d :: path.val];
